@@ -20,13 +20,26 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
+=head1 NAME
+
+mt-aws-glacier - Perl Multithreaded Multipart sync to Amazon Glacier 
+
+=head1 SYNOPSIS
+
+More info in README.md or L<https://github.com/vsespb/mt-aws-glacier> or L<http://mt-aws.com/>
+
+=cut
+
+
 package App::MtAws;
 
 use strict;
 use warnings;
 use utf8;
+use 5.008008; # minumum perl version is 5.8.8
 
-our $VERSION = "0.957beta";
+our $VERSION = "0.961beta";
 
 use constant ONE_MB => 1024*1024;
 
@@ -211,14 +224,22 @@ END
 			
 			my $files = $j->{journal_h};
 			# TODO: refactor
-			my %filelist =	map { $_->{archive_id} => $_ } grep { ! binaryfilename -f $_->{filename} } map { {archive_id => $files->{$_}->{archive_id}, mtime => $files->{$_}{mtime}, relfilename =>$_, filename=> $j->absfilename($_) } } keys %{$files};
+			my %filelist =	map { $_->{archive_id} => $_ }
+				grep { ! binaryfilename -f $_->{filename} }
+				map {
+					{
+						archive_id => $files->{$_}->{archive_id}, mtime => $files->{$_}{mtime}, size => $files->{$_}{size},
+						treehash => $files->{$_}{treehash}, relfilename =>$_, filename=> $j->absfilename($_)
+					}
+				}
+				keys %{$files};
 			if (keys %filelist) {
 				if ($options->{'dry-run'}) {
 					for (values %filelist) {
 						print "Will DOWNLOAD (if available) archive $_->{archive_id} (filename $_->{relfilename})\n"
 					}
 				} else {
-					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::RetrievalFetchJob->new(archives => \%filelist ));
+					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::RetrievalFetchJob->new(file_downloads => $options->{file_downloads}, archives => \%filelist ));
 					my ($R) = fork_engine->{parent_worker}->process_task($ft, $j);
 					die unless $R;
 				}
@@ -313,12 +334,14 @@ Common options:
 	--partsize - Glacier multipart upload part size
 	--filter --include --exclude - File filtering
 	--dry-run - Don't do anything
+	--token - to be used with STS/IAM
 Commands:
 	sync
 		--leaf-optimization - Don't use directory hardlinks count when traverse.
 	purge-vault
 	restore
 	restore-completed
+		--segment-size - Size for multi-segment download, in megabytes
 	check-local-hash
 	retrieve-inventory
 	download-inventory
